/* Program: LDBDBmpw.prg
   Author:  Garry Barrett, Stephen G. Donald
            last updated July 05, 2013
   Purpose: Apply the Lorenz Dominance tests proposed in Barrett, Donald
   and Bhattacharya "Consistent Nonparametric Tests for Lorenz Dominance" (2013)
   JBES, forthcoming.
   
   The test is for the null that Distribution X Lorenz Dominates Distribution Y.
   That is, the test is for H0: LC_x(p) >= LC_y(p) for all p,
                        and H1: LC_x(p) <  LC_y(p) for some p.

   In the notation of Bhattacharya (2003), distribution 1 == x
                                           distribution 2 == y.


Note:
This program for the test of LD is composed of 5procedures
ldtests(y,x)                             @ Constructs the 3 test stats     @
ldboot3(teststat,x,y,brep)               @ LDB3 Emp Bootstrap procedure    @
EmpLor(x)                                @ Calculate the Emp LC            @
LorInteg(gp,LCd)                         @ Integrate the Pos Diff of 2 LCs @
LorInter(gp,LCx)                         @ Interpolates Emp LC ordinates   @



This version allows for weighted data - this is introduced via
 the construction of the empirical lorenz curve.

Assumes Matched-Pair (MP)Sampling - as with panel data.

The data matrix has 3 colunms,
 1 = income or consumption measure A
 2 = income or consumption measure B
 3 = record weight.

 The global variable, pt, indicates if column 1 or 2 represents X in the null.
 Inverse probability weights are assumed. These are rescaled to sum
              to 1.0 in the program


	Last change: GB 29/11/2013 2:45:56 PM
*/

@ Set the file reference for writing the output @;
new;
library pgraph;
graphset;

@   Set Globals @;

@ 1. Set the file reference for inputting the data @

@ ABS  HES data - format of extracts      @
@ col 1=xadj 2=yadj 3=wgt                 @

output file=c:\work\Lorenz\a09cyp.out reset;
print " X = 09c, Y = 09y, Dep";


load x[5009,3]=c:\work\Lorenz\Aus\dat\aus09p.raw;


@ Global Variable. Points to column of data matrix that represents @
@ x in the null hypothesis ...x=1 or 2, y is the converse          @
pt=1;

@ Normalise the total weight for each income vector to 1.0 @
x[.,3]=x[.,3]/sumc(x[.,3]);

mnx1=sumc(x[.,pt].*x[.,3]);
mnx2=sumc(x[.,(3-pt)].*x[.,3]);
sdx1=sumc( (x[.,pt].^2).*x[.,3]) - mnx1^2;
sdx1=sdx1^0.5;

sdx2=sumc( (x[.,(3-pt)].^2).*x[.,3]) - mnx2^2;
sdx2=sdx2^0.5;

nx=rows(x);

@ Set the following globals @
brep=2000;     @ # of bootstrap repetitions to calculate the P-values @

ds=datestr(0);
ts=timestr(0);

print "=====================================================================";
print "Barrett, Bhattacharya and Donald (2007) Tests of Lorenz Dominance";
print "Date " ds "   " ts ;
print "=====================================================================";
print "  ";

et = hsec;       /* record the time length of this run */
rndseed 1364;    /* Set the seed for the random number generator */

{t1,ks2,te}=ldtests(x,pt);    /* construct the test statistic */


{p1,pks2,pe} = ldboot3(t1,ks2,te,x,pt,brep);  /* Bootstrapped P-value for LD */


print "LD Bootstrap Methods";
print "No. of iterations = " brep;
print "Mean X1, SD_x1, n_x =" mnx1~sdx1~nx;
print "Mean X2, SD_x2, n_x =" mnx2~sdx2~nx;
print "  ";
print "KS           test stat = " ks2;
print "             pvalue    = " pks2;
print "  ";
print "CvM          test stat = " t1;
print "             pvalue    = " p1;
print " ";
print "KS equality  test stat = " te;
print "             pvalue    = " pe;
print " ";


/*   */
ds=datestr(0);ts=timestr(0);
print "Date " ds "   " ts ;
et = hsec - et;
print "Time, in seconds" et/100;

output off;
end;

/*====================== Proc: ldtests(x,ptr) =========================*/

proc (3)=ldtests(x,ptr);

  local j,n,m,zx,zy, mnx, mny, pdx, pdy,LCx,LCy,lam,gp,gpc,xx,yy,
        gLCx, gLCy, LCd, indx, indy, kx, ky, pqx, pqy, Ud, U, ld1,ks,kse;

  n = rows(x);  m = n;
  lam = sqrt(n*m/(n+m));
/* construct the empirical LC for x and y at each distinct population quantile */

xx=x[.,ptr]~x[.,3];
yy=x[.,(3-ptr)]~x[.,3];
{LCx} = EmpLor(xx);
{LCy} = EmpLor(yy);

/* Now construct LCs from the empirical LC defined over all empirical population
   proportions defined in either sample

   First - define the set of p, gp, for the comparisons.
         - the expanded LCs are denoted by gLCx and gLCy
*/

gp=unique(LCx[.,1]|LCy[.,1],1);
{gLCx}=LorInter(gp,LCx);
{gLCy}=LorInter(gp,LCy);

/* Take the difference between the gLCs and integrate the positive diff only */

LCd= (gLCy-gLCx);

{Ud} = LorInteg(gp,LCd);

 @ PID test @
 U = lam*sumc(Ud);

 @ KS test    @
 ks  = lam*maxc(LCd);

 @ KS test for LC equality   @
 kse  = lam*maxc(abs(LCd));


@  xy(gp,gLCx~gLCy); @
@  xy(gp,LCd);    @

retp(U,ks,kse);
endp;

/*=============== Proc: ldboot3(testst,ksstat,x,ptr,brep) ===============*/
proc (3)=ldboot3(testst,ks3,kse,x,ptr,brep);

local n,m,xs,ys,j,i,ib,ux,uy,sqn,sqm,lam,prej,pval2,pval3,ksp3,kspe,
LCx,LCy,LCxs,LCys,gp,zgp,gLCx,gLCy,gLCxs,gLCys,LCd,LCds,LCdds,Ud,
Us,tval3,tvale;

  n = rows(x);  m=n;
  lam=sqrt(n*m/(n+m));

prej=zeros(3,1);


{LCx} = EmpLor(x[.,ptr]~x[.,3]);
{LCy} = EmpLor(x[.,(3-ptr)]~x[.,3]);


  for ib (1,brep,1);

/* initialize the vector corresponding to the resampled y */
   ys=zeros(n,2);

/* initialize the vector corresponding to the resampled x */
   xs=zeros(n,2);

/*  resample the x */
   ux=floor(n*rndu(n,1)+ones(n,1));
   for i (1,n,1);
     xs[i,1]=x[ux[i,1],ptr];
     xs[i,2]=x[ux[i,1],3];

     ys[i,1]=x[ux[i,1],(3-ptr)];
     ys[i,2]=x[ux[i,1],3];
   endfor;

@ Normalise the total weight to 1.0 @
ys[.,2]=ys[.,2]/sumc(ys[.,2]);
xs[.,2]=xs[.,2]/sumc(xs[.,2]);

/* Construct the resampled empirical LCs */

{LCxs}=EmpLor(xs);
{LCys}=EmpLor(ys);

gp=unique(LCx[.,1]|LCy[.,1]|LCxs[.,1]|LCys[.,1],1);

{gLCx } = LorInter(gp,LCx);
{gLCy } = LorInter(gp,LCy);
{gLCxs} = LorInter(gp,LCxs);
{gLCys} = LorInter(gp,LCys);

LCd  = gLCy - gLCx;
LCds = gLCys- gLCxs;
LCdds= LCds -LCd;

/* Integrate over the positive difference between the 4 LCs */

{Ud}=LorInteg(gp,LCdds);

@ PID  test stat @
   Us = lam*sumc(Ud);


@ KS test stat     @
   tval3=lam*maxc(LCdds);

@ KS LC equality test stat     @
   tvale=lam*maxc(abs(LCdds));

    if Us > testst;    prej[1] = prej[1]+1;   endif;
    if tval3 > ks3;    prej[2] = prej[2]+1;   endif;
    if tvale > kse;    prej[3] = prej[3]+1;   endif;

  endfor;

  pval3 = prej[1]/brep;
  ksp3  = prej[2]/brep;
  kspe  = prej[3]/brep;

 retp(pval3,ksp3,kspe);

endp;

 /*====================== Proc: EmpLor(x) =========================*/

proc (1)=EmpLor(x);

  /* input  vector x - col.1=income, col.2= pop. proportion */
  /* output vector y - col.1=cum pop share, col.2=cum income share */
  local j,n,zx, mnx,pdx,LCx;

  /* construct the empirical LC for x - LC defined at each distinct population quantile */

n = rows(x);
zx=unique(x[.,1],1);
pdx=rows(zx);
mnx=meanc(x);
LCx=zeros(pdx+1,2);

@ Construct Empirical LCs for X and Y separately @

  for j (1,pdx,1);    /* construct the empirical LC for X  - start at 0 */
      LCx[j+1,1] = sumc((x[.,1] .le zx[j]).*x[.,2])/sumc(x[.,2]);
      LCx[j+1,2] = sumc((x[.,1] .le zx[j]).*x[.,1].*x[.,2])/sumc(x[.,1].*x[.,2]);
  endfor;

retp(LCx);
endp;

 /*====================== Proc: LorInter(gp,LCx) =========================*/

proc (1)=LorInter(gp,LCx);
  local j,n,zgp,gLCx,indx,kx,pqx;


@ Construct LCs from the empirical LC defined for a set of population proportions @
@ using linear interpolation @

@ Input - gp = pre-specificed population proportions @
@       - LCx = Empirical Lorenz curve ... col 1 == pop proportion @
@                                          col 2 == Lor ordinate   @
@                                                                  @
@ Output - gLCx = Lorenz ordinate corresponding to gp              @

n = rows(LCx);
zgp=rows(gp);
gLCx=zeros(zgp,1);

indx=seqa(1,1,n);

for j (2,zgp-1,1);

kx=0; pqx=0;
pqx=maxc((LCx[.,1].le gp[j]).*LCx[.,1]);
kx = maxc(indx.*(LCx[.,1].eq pqx));
if (kx eq (n));
    gLCx[j]=LCx[kx,2];
else;
    gLCx[j]=LCx[kx,2] + ((LCx[kx+1,2]-LCx[kx,2])/(LCx[kx+1,1]-LCx[kx,1]))*(gp[j]-LCx[kx,1]);
endif;

endfor;

gLCx[zgp]=1;

retp(gLCx);
endp;

 /*====================== Proc: LorInteg(gp,LCd) =========================*/

proc (1)=LorInteg(gp,LCd);

  local j,zgp,gpc,Ud;


zgp=rows(gp);
Ud=zeros(zgp,1);

for j (2,zgp-1,1);

/* Doing the integration for each segment of the LCs sequentially */

@ the area between the LCs for the first segment is a triangle @
if (j eq 2);
   if (LCd[j] gt 0);
       Ud[j] = gp[j]*LCd[j]/2;
   endif;
else;

@ now deal with the next series of segments which may a trapezium if there is no crossing on the interval @
@ or a triangle if there is an crossing on the interval - deal with each of the cases in turn             @

   if ((LCd[j] ge 0) and (LCd[j-1] ge 0));

@ this first branch deals with weak dominance over the interval @

       Ud[j] = (gp[j]-gp[j-1])*(LCd[j]+LCd[j-1])/2;


   elseif ((LCd[j] lt 0) and (LCd[j-1] gt 0));

@ this second branch deals with LCy crossing LCx from above           @
@ need to determine the population proportion where the LCs intersect @
@ then determine the area of the triangle                             @

   gpc= -(LCd[j-1])*(gp[j]-gp[j-1])/(LCd[j]-LCd[j-1]) ;

   Ud[j] =  (LCd[j-1])*gpc/2;


   elseif ((LCd[j] gt 0) and (LCd[j-1] lt 0));

@ this third branch deals with LCy crossing LCx from below            @
@ need to determine the population proportion where the LCs intersect @
@ then determine the area of the triangle                             @

   gpc= -(LCd[j-1])*(gp[j]-gp[j-1])/(LCd[j]-LCd[j-1]) ;

   Ud[j] =  (LCd[j])*(gp[j]-gp[j-1]- gpc)/2;

  endif;
endif;

endfor;

@ Now deal with the last segment of the LCs and the last piece of integration @


  if (LCd[zgp-1] gt 0);
      Ud[zgp]= LCd[zgp-1]*(1-gp[zgp-1])/2;
  endif;

retp(Ud);
endp;
